@@ -17,7 +17,6 @@
 from __future__ import annotations
 
 import math
-from typing import Optional, Tuple, Union
 
 import tensorflow as tf
 
@@ -90,7 +89,7 @@ class TFSegformerOverlapPatchEmbeddings(keras.layers.Layer):
         self.num_channels = num_channels
         self.hidden_size = hidden_size
 
-    def call(self, pixel_values: tf.Tensor) -> Tuple[tf.Tensor, int, int]:
+    def call(self, pixel_values: tf.Tensor) -> tuple[tf.Tensor, int, int]:
         embeddings = self.proj(self.padding(pixel_values))
         height = shape_list(embeddings)[1]
         width = shape_list(embeddings)[2]
@@ -115,7 +114,7 @@ class TFSegformerOverlapPatchEmbeddings(keras.layers.Layer):
 
 class TFSegformerEfficientSelfAttention(keras.layers.Layer):
     """SegFormer's efficient self-attention mechanism. Employs the sequence reduction process introduced in the [PvT
-    paper](https://arxiv.org/abs/2102.12122)."""
+    paper](https://huggingface.co/papers/2102.12122)."""
 
     def __init__(
         self,
@@ -169,7 +168,7 @@ class TFSegformerEfficientSelfAttention(keras.layers.Layer):
         width: int,
         output_attentions: bool = False,
         training: bool = False,
-    ) -> Union[tf.Tensor, Tuple[tf.Tensor, tf.Tensor]]:
+    ) -> tf.Tensor | tuple[tf.Tensor, tf.Tensor]:
         batch_size = shape_list(hidden_states)[0]
         num_channels = shape_list(hidden_states)[2]
 
@@ -272,7 +271,7 @@ class TFSegformerAttention(keras.layers.Layer):
 
     def call(
         self, hidden_states: tf.Tensor, height: int, width: int, output_attentions: bool = False
-    ) -> Union[tf.Tensor, Tuple[tf.Tensor, tf.Tensor]]:
+    ) -> tf.Tensor | tuple[tf.Tensor, tf.Tensor]:
         self_outputs = self.self(hidden_states, height, width, output_attentions)
 
         attention_output = self.dense_output(self_outputs[0])
@@ -325,8 +324,8 @@ class TFSegformerMixFFN(keras.layers.Layer):
         self,
         config: SegformerConfig,
         in_features: int,
-        hidden_features: int = None,
-        out_features: int = None,
+        hidden_features: int | None = None,
+        out_features: int | None = None,
         **kwargs,
     ):
         super().__init__(**kwargs)
@@ -401,7 +400,7 @@ class TFSegformerLayer(keras.layers.Layer):
         width: int,
         output_attentions: bool = False,
         training: bool = False,
-    ) -> Tuple:
+    ) -> tuple:
         self_attention_outputs = self.attention(
             self.layer_norm_1(hidden_states),  # in Segformer, layernorm is applied before self-attention
             height,
@@ -499,11 +498,11 @@ class TFSegformerEncoder(keras.layers.Layer):
     def call(
         self,
         pixel_values: tf.Tensor,
-        output_attentions: Optional[bool] = False,
-        output_hidden_states: Optional[bool] = False,
-        return_dict: Optional[bool] = True,
+        output_attentions: bool | None = False,
+        output_hidden_states: bool | None = False,
+        return_dict: bool | None = True,
         training: bool = False,
-    ) -> Union[Tuple, TFBaseModelOutput]:
+    ) -> tuple | TFBaseModelOutput:
         all_hidden_states = () if output_hidden_states else None
         all_self_attentions = () if output_attentions else None
 
@@ -580,11 +579,11 @@ class TFSegformerMainLayer(keras.layers.Layer):
     def call(
         self,
         pixel_values: tf.Tensor,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
         training: bool = False,
-    ) -> Union[Tuple, TFBaseModelOutput]:
+    ) -> tuple | TFBaseModelOutput:
         output_attentions = output_attentions if output_attentions is not None else self.config.output_attentions
         output_hidden_states = (
             output_hidden_states if output_hidden_states is not None else self.config.output_hidden_states
@@ -609,7 +608,7 @@ class TFSegformerMainLayer(keras.layers.Layer):
 
         # Change the other hidden state outputs to NCHW as well
         if output_hidden_states:
-            hidden_states = tuple([tf.transpose(h, perm=(0, 3, 1, 2)) for h in encoder_outputs[1]])
+            hidden_states = tuple(tf.transpose(h, perm=(0, 3, 1, 2)) for h in encoder_outputs[1])
 
         if not return_dict:
             if tf.greater(len(encoder_outputs[1:]), 0):
@@ -666,7 +665,7 @@ SEGFORMER_START_DOCSTRING = r"""
 SEGFORMER_INPUTS_DOCSTRING = r"""
 
     Args:
-        pixel_values (`np.ndarray`, `tf.Tensor`, `List[tf.Tensor]` ``Dict[str, tf.Tensor]` or `Dict[str, np.ndarray]` and each example must have the shape `(batch_size, num_channels, height, width)`):
+        pixel_values (`np.ndarray`, `tf.Tensor`, `list[tf.Tensor]` ``dict[str, tf.Tensor]` or `dict[str, np.ndarray]` and each example must have the shape `(batch_size, num_channels, height, width)`):
             Pixel values. Pixel values can be obtained using [`AutoImageProcessor`]. See
             [`SegformerImageProcessor.__call__`] for details.
 
@@ -714,11 +713,11 @@ class TFSegformerModel(TFSegformerPreTrainedModel):
     def call(
         self,
         pixel_values: tf.Tensor,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
         training: bool = False,
-    ) -> Union[Tuple, TFBaseModelOutput]:
+    ) -> tuple | TFBaseModelOutput:
         outputs = self.segformer(
             pixel_values,
             output_attentions=output_attentions,
@@ -767,10 +766,10 @@ class TFSegformerForImageClassification(TFSegformerPreTrainedModel, TFSequenceCl
         self,
         pixel_values: tf.Tensor | None = None,
         labels: tf.Tensor | None = None,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
-    ) -> Union[Tuple, TFSequenceClassifierOutput]:
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
+    ) -> tuple | TFSequenceClassifierOutput:
         outputs = self.segformer(
             pixel_values,
             output_attentions=output_attentions,
@@ -951,10 +950,10 @@ class TFSegformerForSemanticSegmentation(TFSegformerPreTrainedModel):
         self,
         pixel_values: tf.Tensor,
         labels: tf.Tensor | None = None,
-        output_attentions: Optional[bool] = None,
-        output_hidden_states: Optional[bool] = None,
-        return_dict: Optional[bool] = None,
-    ) -> Union[Tuple, TFSemanticSegmenterOutput]:
+        output_attentions: bool | None = None,
+        output_hidden_states: bool | None = None,
+        return_dict: bool | None = None,
+    ) -> tuple | TFSemanticSegmenterOutput:
         r"""
         labels (`tf.Tensor` of shape `(batch_size, height, width)`, *optional*):
             Ground truth semantic segmentation maps for computing the loss. Indices should be in `[0, ...,
